﻿using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ZhonTai.Admin.Core.Consts;
using ZhonTai.Admin.Core.Dto;
using ZhonTai.Admin.Domain.Banner;
using ZhonTai.Admin.Services.Banner.Dto;
using ZhonTai.DynamicApi;
using ZhonTai.DynamicApi.Attributes;

namespace ZhonTai.Admin.Services.Banner
{
    /// <summary>
    /// 轮播图服务
    /// </summary>
    [DynamicApi(Area = AdminConsts.AreaName)]
    public class BannerService : BaseService, IBannerService, IDynamicApi
    {
        private IBannerRepository _bannerRepository => LazyGetRequiredService<IBannerRepository>();

        public BannerService()
        {
        }

        /// <summary>
        /// 查询模块
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<BannerGetOutput> GetAsync(long id)
        {
            var result = await _bannerRepository.GetAsync<BannerGetOutput>(id);
            return result;
        }

        /// <summary>
        /// 查询分页
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<BannerListOutput>> GetPageAsync(PageInput<BannerGetPageDto> input)
        {
            // 0待上架 1上架 2下架
            var status = input.Filter?.Status;
            DateTime dtnow = DateTime.Now;

            var list = await _bannerRepository.Select
            .WhereIf(status.HasValue && status >= 0, a => a.Status == status)            
            .Count(out var total)
            .OrderByDescending(true, c => c.Rank)
            .OrderByDescending(true, c => c.Id)
            .Page(input.CurrentPage, input.PageSize)
            .ToListAsync<BannerListOutput>();

            var data = new PageOutput<BannerListOutput>()
            {
                List = list,
                Total = total
            };

            return data;
        }

        /// <summary>
        /// 新增
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<long> AddAsync(BannerAddInput input)
        {
            var entity = Mapper.Map<BannerEntity>(input);            
            await _bannerRepository.InsertAsync(entity);

            return entity.Id;
        }

        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task UpdateAsync(BannerUpdateInput input)
        {
            var entity = await _bannerRepository.GetAsync(input.Id);
            if (!(entity?.Id > 0))
            {
                throw ResultOutput.Exception("轮播图信息不存在");
            }

            Mapper.Map(input, entity);
            await _bannerRepository.UpdateAsync(entity);
        }        

        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task SoftDeleteAsync(long id)
        {
            await _bannerRepository.SoftDeleteAsync(id);
        }
        /// <summary>
        /// 修改状态
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet]
        public async Task UpdateStatus(long id,int status)
        {
            if (id <= 0 || !new[] { 1,2}.Contains(status)) {
                throw ResultOutput.Exception("缺少有效参数，请刷新后重试");
            }
            var entity = await _bannerRepository.GetAsync(id);
            if (!(entity?.Id > 0))
            {
                throw ResultOutput.Exception("轮播图信息不存在");
            }
            if (entity.Status == status) {
                throw ResultOutput.Exception("无效操作请刷新后重试");
            }
            await _bannerRepository.UpdateDiy.Set(a => a.Status, status).Where(m => m.Id == id).ExecuteAffrowsAsync();                        
        }
    }
}